/****************************************************************************************
 * 
 * Papi32mc.h - Papi32mc.dll header file - POS ( s509 ) API for Windows NT 3.51/4.0
 * 
 * Copyright(c) Sangoma Technologies Inc. 1997
 *
 * Author:	Anton Belov
 * 
 * Application include file for s509 constants, command codes, data structures and
 * Papi32mc.dll functions. See function descriptions in this file for a complete 
 * function reference.
 *
 * Must #include <windows.h> before this header for type definitions used here.
 *
 * $Log:   C:/anton/pos/papi32mc/pvcs/papi32mc.h_v  $
 * 
 *    Rev 1.3   27 Feb 1997 10:57:28   Anton
 * Notes about thread re-entrace removed, because it wasn't true
 * 
 *    Rev 1.2   24 Feb 1997 19:33:50   Anton
 * Control Block header length is 16 and not 15 !!
 * 
 *    Rev 1.1   24 Feb 1997 09:46:02   Anton
 * PAPI_ERROR() and PAPI_OK() macros added
 * 
 *    Rev 1.0   21 Feb 1997 15:10:18   Anton
 * Initial revision.
 * 
 */

//
//***********************************   Constants   *************************************
//

// POS mailbox offsets from the begining of shared memory window - see S509 POS card
// manual for details
#define OP_FLAG_OFF				0x0E000 // execution flag
#define POS_STATE_OFF			0x0E001	// POS Port 1,2 state
#define ASYNC_STATE_OFF			0x0E002	// Asynch. Port 1,2 state
#define CBLOCK_OFF				0x0E010	// Begining of Control Block

// Bit masks for status bytes - see S509 POS card manual for details
#define POS_RX_DATA_AVAIL_1		0x01	// Rx data available on POS Port 1
#define POS_RX_DATA_AVAIL_2		0x02	// Rx data available on POS Port 2
#define ASYNC_RX_DATA_AVAIL_1	0x01	// Rx data available on Async. Port 1
#define ASYNC_TX_BUFF_EMPTY_1	0x02	// Tx buffer for Async. Port 1 is empty
#define ASYNC_RX_DATA_AVAIL_2	0x04	// Rx data available on Async. Port 2
#define ASYNC_TX_BUFF_EMPTY_2	0x08	// Tx buffer for Async. Port 2 is empty

// Command Codes - see S509 POS card manual for details
#define CONFIGURE				0x01
#define SEND_ASYNC				0x02
#define RECEIVE_ASYNC			0x03
#define ENABLE_POS				0x04
#define DISABLE_POS				0x05
#define RECEIVE_POS				0x06
#define READ_ERROR_STATISTICS	0x07
#define FLUSH_ERROR_STATISTICS	0x08
#define GET_STATUS				0x09
#define POS_SETUP				0x0A
#define RESET_CARD				0x0B

// Card Return codes - see S509 POS card manual for details
#define RCODE_SUCCESS			0x00
#define RCODE_WRONG_PORT		0x01
#define RCODE_WRONG_STATE		0x02
#define RCODE_PORT_DISABLED		0x03
#define RCODE_BUSY				0x33
#define RCODE_INV_COMMAND		0x80
#define RCODE_INV_PARAM			0x81

// Line speed on Async ports - to be used with CONFIGURE command
// - see S509 POS card manual for details
#define	LS_4800					0x01	// 4800	 bps
#define LS_9600					0x02	// 9600  bps
#define LS_19200				0x03	// 19200 bps
#define LS_38400				0x04	// 38400 bps

// Async port mode - to be used with CONFIGURE command
// - see S509 POS card manual for details
#define MD_HALF_DPLX			0x00	// Half-Duplex
#define MD_FULL_DPLX			0x01	// Full-Duplex

// Control Block constants- see S509 POS card manual for details
#define	CB_DATALEN				1030	// Control Block data field length
#define	CB_HEADERLEN			0x10	// Control Block header length

//
// PAPI_STATUS return codes
//
// Note: non-negative return codes mean various kinds of success
//		 negative return code mean various kinds of failure
//

// success codes - see function descriptions for details
#define	PAPI_STATUS_SUCCESS			0
#define PAPI_STATUS_OPENED			1
#define PAPI_STATUS_CLOSED			2
#define PAPI_STATUS_REGISTERED		3
#define PAPI_STATUS_DEREGISTERED	4

// failure codes - see function descriptions for details
#define PAPI_STATUS_FAILURE			-1
#define PAPI_STATUS_HANDLE			-2
#define PAPI_STATUS_BLOCKED			-3
#define PAPI_STATUS_Z80_TIMEOUT		-4

//				
//
//**************************  Macros  *****************************
//
#define PAPI_ERROR( _status_ )		( _status_ < 0 )		// TRUE if PAPI_STATUS_ ... code
															// indicates error
#define PAPI_OK( _status_ )			( _status_ >= 0 )		// TRUE if PAPI_STATUS_ ... code
															// indicates success
//				
//
//**************************  Types and Data Structures  *****************************
//
#define PAPI_STATUS				int		// Status returned by the calls
#define SANGOMAIMPORT 
#ifdef IN								// make sure that IN defined ot nothing
#undef IN
#define IN
#endif
#ifdef OUT								// make sure that OUT defined ot nothing
#undef OUT
#define OUT
#endif


// Control Block structure - see S509 POS card manual for details
// Note: structure must be packed on 1-byte boundaries and for a block this size, 
//		 it is not wise to allocate it on the stack - should be a static global.
#pragma pack(1)
typedef struct {
	BYTE command;			// Command code - see Command Codes above
	WORD buffer_length;		// Length of data buffer associated with this call
	BYTE return_code;		// Return code - see Card Return codes
	BYTE port_number;		// POS or Async port number
	BYTE attribute;			// Attribute byte
	BYTE reserved[10];		// Reserved for later use
	BYTE data[CB_DATALEN];	// Data field
} CBLOCK, *PCBLOCK;
#pragma pack()


//
//*************************  Imported functions prototypes  **************************
//

#if defined (__cplusplus)
extern "C" {
#endif

//
// PAPIAlive() - always returns PAPI_STATUS_SUCCESS
//
//	Input					: none
//	Output					: none
//	Return codes on success : PAPI_STATUS_SUCCESS always
//	Return codes on failure : never fails
//
SANGOMAIMPORT PAPI_STATUS PAPIAlive(
									void
									);

//
// PAPISetup() - Opens posdla device.
//
//	Input					: none
//	Output					: none
//	Return codes on success	: PAPI_STATUS_SUCCESS - the device has been successfully opened, and 
//													this is the first call done by the application
//													instance to this fucntion.
//							  PAPI_STATUS_OPENED  - the device has been already opened, probably 
//													by some other thread of this application 
//													instance.
//	Return codes on failure : PAPI_STATUS_FAILURE - couldn't open the posdla device. Make sure
//													that the POSDLA.SYS driver is loaded.
//
SANGOMAIMPORT PAPI_STATUS PAPISetup(
									void
									);

//
// PAPICleanup() - Closes posdla device.
//
//	Input					: none
//	Output					: none
//	Return codes on success	: PAPI_STATUS_SUCCESS - the device has been successfully closed.
//							  PAPI_STATUS_CLOSED  - the device has been already closed. This will
//													happen if you call this function more than 
//													once in a row. Nothing bad happens, but still..
//													what are you doing?
//	Return codes on failure : PAPI_STATUS_FAILURE - couldn't close the posdla device. I'm not sure
//													this is possible unless something wrong
//													happened to the system
//
SANGOMAIMPORT PAPI_STATUS PAPICleanup(
									  void
									  );

//
// PAPIRegisterCard() - Registers given card with posdla device.
//
//	Input					: cardnumber - card number to register
//	Output					: none
//	Return codes on success	: PAPI_STATUS_SUCCESS    - the card has been successfully registered with 
//													   the device and is the first registration call 
//													   for this card
//							  PAPI_STATUS_REGISTERED - the card has been successfully registered with 
//													   the device and this is not the first 
//													   registration call for this card. In most of
//													   cases this means that some other application
//													   that works with the card is currently running,
//													   so you'd better know what are you doing.
//	Return codes on failure : PAPI_STATUS_FAILURE	 - couldn't register the card with the device.
//													   This mostly happens if there are some memory
//													   problems - you may retry later, or restart
//													   the driver.
//							  PAPI_STATUS_HANDLE	 - couldn't talk to the driver. Make sure that
//													   PAPISetup() call was successfully performed.
//
//
SANGOMAIMPORT PAPI_STATUS PAPIRegisterCard(
										   IN BYTE cardnumber
										   );

//
// PAPIDeregisterCard() - Deregisters given card with posdla device.
//
//	Input					: cardnumber - card number to deregister
//	Output					: none
//	Return codes on success	: PAPI_STATUS_SUCCESS    - the card has been successfully deregistered with 
//													   the device and there are no more applications
//													   registered for this card.
//							  PAPI_STATUS_DEREGISTERED-the card has been successfully deregistered 
//													   with the device, however there are more 
//													   applications currently registered with this
//													   card.
//	Return codes on failure : PAPI_STATUS_FAILURE	 - couldn't deregister the card with the device.
//													   You may retry later, or just ignore it.
//							  PAPI_STATUS_HANDLE	 - couldn't talk to the driver. Make sure that
//													   PAPISetup() call was successfully performed.
//
//
SANGOMAIMPORT PAPI_STATUS PAPIDeregisterCard(
											 IN BYTE cardnumber
											 );

//
// PAPIGetCards() - Returns information about registered and configured cards.
//
//	Input					: none
//	Output					: *registeredCards - bit mask of registered cards. Bit i-th set if card
//												 number i is registered with the driver ( the 
//												 application that registered the card is not 
//												 nessesary the calling one, i.e. it is possible
//												 to perform PAPIGetCards() before PAPIRegisterCard(0)
//												 for expample and receive that card 0 is registered.
//							  *configuredCards - bit mask of all configured cards. Same as above.
//
//	Return codes on success	: PAPI_STATUS_SUCCESS    - the call has been successfully performed and 
//													   cards information available in 
//													   *registeredCards and *configuredCards.
//	Return codes on failure : PAPI_STATUS_FAILURE	 - couldn't obtain card information.
//													   You may retry later.
//							  PAPI_STATUS_HANDLE	 - couldn't talk to the driver. Make sure that
//													   PAPISetup() call was successfully performed.
//
SANGOMAIMPORT PAPI_STATUS PAPIGetCards(
									   OUT PUSHORT registeredCards, 
									   OUT PUSHORT configuredCards
									   );

//
// PAPIControl() - Performs Control Block command on give card. The function passes given 
//				   Control Block "as is" to the board, set the Op Flag to 1, waits for it
//				   to be cleared by the code running on board and passes updated Control Block
//				   from the board back to the application. The Control Block operations are 
//				   protected by mutex. The maximum time to wait for this mutex is defined by
//				   CONTOL_MUTEX_TIMEOUT	and currentrly set to 3 seconds. The mamimum time to
//				   wait for the Op Flag to be cleared is defined by COMMAND_TIMEOUT and currently
//				   set to 2 seconds. If this time expires the application will receive appropriate
//				   error and the most probeble reason for this timeout is that the code on board
//				   is hang - the only way to fix it is to reload the board.
//
//	Input					: cBlock		- Contol Block containing the board command and it's 
//											  parameters. See S509 POS manual for details.
//							  cardnumber	- card on which the command will be performed
//
//	Output					: cBlock		- in case PAPI_STATUS_SUCCESS returned this will
//											  be the Control Block as returned by the board,
//											  containing the command result.
//
//	Return codes on success	: PAPI_STATUS_SUCCESS    - the operation has been successfully performed
//													   in terms of the driver and resulting Control
//													   Block will be returned in *cBlock
//	Return codes on failure : PAPI_STATUS_FAILURE	 - Internal error occured. You may retry later.
//							  PAPI_STATUS_HANDLE	 - couldn't talk to the driver. Make sure that
//													   PAPISetup() call was successfully performed.
//							  PAPI_STATUS_BLOCKED	 - the Control Block currently blocked by some
//													   other process - CONTOL_MUTEX_TIMEOUT expired
//							  PAPI_STATUS_Z80_TIMEOUT- the Op Flag hasn't been cleared within
//													   COMMAND_TIMEOUT period - probably the code
//													   on board is not running or hung. This also
//													   will happen if the code is not loaded.
//
SANGOMAIMPORT PAPI_STATUS PAPIControl(
									  IN OUT PCBLOCK cBlock, 
									  IN BYTE cardnumber
									  );


//*********************************************************************************************
// Low level DLL exports - you'd better know what are you doing ...
//

//
// PAPIReadPortByte() - Reads one byte from the cards i/o port
//
//	Input					: cardnumber - card number
//							  offset	 - offset of byte to be read from cards base i/o address
//
//	Output					: *data		 - byte read in case PAPI_STATUS_SUCCESS returned
//	Return codes on success	: PAPI_STATUS_SUCCESS    - the command was successfully performed and
//													   the byte read is available in *data
//	Return codes on failure : PAPI_STATUS_FAILURE	 - Internal error occured. You may retry later.
//							  PAPI_STATUS_HANDLE	 - couldn't talk to the driver. Make sure that
//													   PAPISetup() call was successfully performed.
//
SANGOMAIMPORT PAPI_STATUS PAPIReadPortByte(
										   IN BYTE cardnumber, 
										   IN WORD offset, 
										   OUT BYTE* data
										   );

//
// PAPIWritePortByte() - Writes one byte to the cards i/o port
//
//	Input					: cardnumber - card number
//							  offset	 - offset of byte to write from cards base i/o address
//							  data		 - byte to write
//	Output					: none
//	Return codes on success	: PAPI_STATUS_SUCCESS    - the command was successfully performed
//	Return codes on failure : PAPI_STATUS_FAILURE	 - Internal error occured. You may retry later.
//							  PAPI_STATUS_HANDLE	 - couldn't talk to the driver. Make sure that
//													   PAPISetup() call was successfully performed.
//
SANGOMAIMPORT PAPI_STATUS PAPIWritePortByte(
											IN BYTE cardnumber, 
											IN WORD offset, 
											IN BYTE data
											);

//
// PAPIReadByte() - Reads one byte from the cards memory
//
//	Input					: cardnumber - card number
//							  offset	 - offset of byte to be read from cards shared memory
//										   window base
//	Output					: *data		 - byte read in case PAPI_STATUS_SUCCESS returned
//	Return codes on success	: PAPI_STATUS_SUCCESS    - the command was successfully performed and
//													   the byte read is available in *data
//	Return codes on failure : PAPI_STATUS_FAILURE	 - Internal error occured. You may retry later.
//							  PAPI_STATUS_HANDLE	 - couldn't talk to the driver. Make sure that
//													   PAPISetup() call was successfully performed.
//
SANGOMAIMPORT PAPI_STATUS PAPIReadByte(
									   IN BYTE cardnumber, 
									   IN WORD offset, 
									   OUT BYTE* data
									   );

//
// PAPIWriteByte() - Writes one byte to the cards memory
//
//	Input					: cardnumber - card number
//							  offset	 - offset from cards shared memory window base
//										   of byte to write
//							  data		 - byte to write
//	Output					: none
//	Return codes on success	: PAPI_STATUS_SUCCESS    - the command was successfully performed
//	Return codes on failure : PAPI_STATUS_FAILURE	 - Internal error occured. You may retry later.
//							  PAPI_STATUS_HANDLE	 - couldn't talk to the driver. Make sure that
//													   PAPISetup() call was successfully performed.
//
SANGOMAIMPORT PAPI_STATUS PAPIWriteByte(
										IN BYTE cardnumber, 
										IN WORD offset, 
										IN BYTE data
										);

//
// PAPIReadBlock() - Reads block of given size from the cards memory
//
//	Input					: cardnumber - card number
//							  offset	 - offset of the begining of the block to be read from cards
//										   shared memory window base
//							  length	 - size of the block in bytes - at least 1
//	Output					: *data		 - block read in case PAPI_STATUS_SUCCESS returned - the 
//										   memory for the block assumed to be allocated.
//	Return codes on success	: PAPI_STATUS_SUCCESS    - the command was successfully performed and
//													   the block read is available in *data
//	Return codes on failure : PAPI_STATUS_FAILURE	 - Internal error occured. You may retry later.
//							  PAPI_STATUS_HANDLE	 - couldn't talk to the driver. Make sure that
//													   PAPISetup() call was successfully performed.
//
SANGOMAIMPORT PAPI_STATUS PAPIReadBlock(
										IN BYTE cardnumber, 
										IN WORD offset, 
										IN WORD length, 
										OUT BYTE* data
										);

//
// PAPIWriteBlock() - Writes data block to the cards memory
//
//	Input					: cardnumber - card number
//							  offset	 - offset from cards shared memory window base
//										   of block to write
//							  length	 - size of the block in bytes - at least 1
//							  data		 - block to write
//	Output					: none
//	Return codes on success	: PAPI_STATUS_SUCCESS    - the command was successfully performed
//	Return codes on failure : PAPI_STATUS_FAILURE	 - Internal error occured. You may retry later.
//							  PAPI_STATUS_HANDLE	 - couldn't talk to the driver. Make sure that
//													   PAPISetup() call was successfully performed.
//
SANGOMAIMPORT PAPI_STATUS PAPIWriteBlock(
										 IN BYTE cardnumber, 
										 IN WORD offset, 
										 IN WORD length, 
										 IN BYTE* data
										 );

SANGOMAIMPORT PAPI_STATUS PAPIStopS514Card(IN BYTE cardnumber);
SANGOMAIMPORT PAPI_STATUS PAPIStartS514Card(IN BYTE cardnumber);

#ifdef __cplusplus
}
#endif
